const express = require('express');
const fs = require('fs');
const path = require('path');
const bodyParser = require('body-parser');
const axios = require('axios'); // For making HTTP requests to OpenRouter

const app = express();
const PORT = 9200; // Port for the backend server

// Middleware to parse JSON and form data
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }));

// Serve the frontend files (HTML, CSS, JS)
app.use(express.static(path.join(__dirname, 'public')));

// Path to the JSON files
const chatHistoryFilePath = path.join(__dirname, 'chatHistory.json');
const trainingFilePath = path.join(__dirname, 'trainingData.json');
const jarkovChainFile = path.join(__dirname, 'jarkovChain.json');

// Initialize the JSON files if they don't exist
if (!fs.existsSync(chatHistoryFilePath)) {
    fs.writeFileSync(chatHistoryFilePath, JSON.stringify([], null, 2));
}
if (!fs.existsSync(trainingFilePath)) {
    fs.writeFileSync(trainingFilePath, JSON.stringify([], null, 2));
}
if (!fs.existsSync(jarkovChainFile)) {
    fs.writeFileSync(jarkovChainFile, JSON.stringify({}, null, 2));
}

// Load the Markov chain
let jarkovChain = {};
if (fs.existsSync(jarkovChainFile)) {
    jarkovChain = JSON.parse(fs.readFileSync(jarkovChainFile, 'utf8'));
}

// Tokenizer function
function tokenize(text) {
    return text
        .toLowerCase()
        .replace(/[^\w\s]/g, ' ') // Remove punctuation
        .split(/\s+/); // Split into words
}

// Function to build a two-dimensional Markov chain
function buildjarkovChain(text) {
    const words = tokenize(text);
    const chain = {};

    for (let i = 0; i < words.length - 1; i++) {
        const currentWord = words[i];
        const nextWord = words[i + 1];

        if (!chain[currentWord]) {
            chain[currentWord] = {};
        }

        if (!chain[currentWord][nextWord]) {
            chain[currentWord][nextWord] = 1; // Initialize frequency
        } else {
            chain[currentWord][nextWord]++; // Increment frequency
        }
    }

    return chain;
}

// Function to update the Markov chain with new data
function updatejarkovChain(newChain) {
    for (const [key, value] of Object.entries(newChain)) {
        if (!jarkovChain[key]) {
            jarkovChain[key] = {};
        }
        for (const [nextWord, freq] of Object.entries(value)) {
            if (!jarkovChain[key][nextWord]) {
                jarkovChain[key][nextWord] = freq;
            } else {
                jarkovChain[key][nextWord] += freq;
            }
        }
    }
}

// Function to generate a Markov response
function generateMarkovResponse(input, chain) {
    const words = tokenize(input);
    let currentWord = words[Math.floor(Math.random() * words.length)] || '';

    // If the selected word doesn't exist in the chain, pick a random word from the chain
    if (!chain[currentWord]) {
        const chainWords = Object.keys(chain);
        currentWord = chainWords[Math.floor(Math.random() * chainWords.length)];
    }

    let response = [currentWord];

    for (let i = 0; i < 100; i++) { // Increased loop limit for longer responses
        if (chain[currentWord]) {
            // Get all possible next words and their frequencies
            const nextWords = Object.keys(chain[currentWord]);
            const frequencies = Object.values(chain[currentWord]);

            // Calculate total frequency
            const totalFrequency = frequencies.reduce((sum, freq) => sum + freq, 0);

            // Randomly select the next word based on frequency
            let randomValue = Math.random() * totalFrequency;
            let selectedWord = '';

            for (let j = 0; j < nextWords.length; j++) {
                randomValue -= frequencies[j];
                if (randomValue <= 0) {
                    selectedWord = nextWords[j];
                    break;
                }
            }

            currentWord = selectedWord;
            response.push(currentWord);
        } else {
            // Fallback: Pick a random word from the chain
            const chainWords = Object.keys(chain);
            currentWord = chainWords[Math.floor(Math.random() * chainWords.length)];
            response.push(currentWord);
        }
    }

    return response.join(' ');
}

// Endpoint to generate a response and update the Markov chain
app.post('/generate-response', (req, res) => {
    const { prompt } = req.body;

    if (!prompt) {
        return res.status(400).send('Prompt is required.');
    }

    // Generate a response using the Markov chain
    const response = generateMarkovResponse(prompt, jarkovChain);

    // Update the Markov chain with the new prompt and response
    const promptChain = buildjarkovChain(prompt);
    const responseChain = buildjarkovChain(response);

    updatejarkovChain(promptChain);
    updatejarkovChain(responseChain);

    // Save the updated Markov chain to the file
    fs.writeFileSync(jarkovChainFile, JSON.stringify(jarkovChain, null, 2));

    // Send the response back to the frontend
    res.status(200).json({ response });
});

// Endpoint to save chat history
app.post('/save-chat', (req, res) => {
    const { username, prompt, response } = req.body;

    if (!username || !prompt || !response) {
        return res.status(400).send('Username, prompt, and response are required.');
    }

    // Read the existing chat history
    fs.readFile(chatHistoryFilePath, 'utf8', (err, data) => {
        if (err) {
            console.error('Error reading file:', err);
            return res.status(500).send('Error reading chat history.');
        }

        // Parse the existing data
        let chatHistory = [];
        try {
            chatHistory = JSON.parse(data);
        } catch (parseError) {
            console.error('Error parsing JSON:', parseError);
            return res.status(500).send('Error parsing chat history.');
        }

        // Add the new chat entry
        chatHistory.push({
            username,
            prompt,
            response,
            timestamp: new Date().toISOString(), // Add a timestamp
        });

        // Write the updated chat history back to the JSON file
        fs.writeFile(chatHistoryFilePath, JSON.stringify(chatHistory, null, 2), (writeErr) => {
            if (writeErr) {
                console.error('Error writing file:', writeErr);
                return res.status(500).send('Error saving chat history.');
            }

            res.status(200).send('Chat saved successfully.');
        });
    });
});

// Endpoint to load chat history
app.get('/load-chat', (req, res) => {
    // Read the existing chat history
    fs.readFile(chatHistoryFilePath, 'utf8', (err, data) => {
        if (err) {
            console.error('Error reading file:', err);
            return res.status(500).send('Error reading chat history.');
        }

        // Parse the existing data
        let chatHistory = [];
        try {
            chatHistory = JSON.parse(data);
        } catch (parseError) {
            console.error('Error parsing JSON:', parseError);
            return res.status(500).send('Error parsing chat history.');
        }

        // Send the chat history back to the frontend
        res.status(200).json(chatHistory);
    });
});

// Endpoint to handle training data
// Endpoint to handle training data
app.post('/train', (req, res) => {
    const { prompt, response } = req.body;

    if (!prompt || !response) {
        return res.status(400).send('Prompt and response are required.');
    }

    // Read the existing training data
    fs.readFile(trainingFilePath, 'utf8', (err, data) => {
        if (err) {
            console.error('Error reading file:', err);
            return res.status(500).send('Error reading training data.');
        }

        // Parse the existing data
        let trainingData = [];
        try {
            trainingData = JSON.parse(data);
        } catch (parseError) {
            console.error('Error parsing JSON:', parseError);
            return res.status(500).send('Error parsing training data.');
        }

        // Add the new training data
        trainingData.push({
            prompt,
            response,
            timestamp: new Date().toISOString(), // Add a timestamp
        });

        // Write the updated training data back to the JSON file
        fs.writeFile(trainingFilePath, JSON.stringify(trainingData, null, 2), (writeErr) => {
            if (writeErr) {
                console.error('Error writing file:', writeErr);
                return res.status(500).send('Error saving training data.');
            }

            // Update the Markov chain with the new training data
            const promptChain = buildjarkovChain(prompt);
            const responseChain = buildjarkovChain(response);

            updatejarkovChain(promptChain);
            updatejarkovChain(responseChain);

            // Save the updated Markov chain to the file
            fs.writeFileSync(jarkovChainFile, JSON.stringify(jarkovChain, null, 2));

            res.status(200).send('Training data saved successfully.');
        });
    });
});

// Start the server
app.listen(PORT, '0.0.0.0', () => {
    console.log(`Server is running on http://thomaslopezforte.searchcontent.tech:${PORT}`);
});