document.addEventListener('DOMContentLoaded', () => {
    const chatBox = document.getElementById('chat-box');
    const userInput = document.getElementById('user-input');
    const sendButton = document.getElementById('send-button');
    const fileInput = document.getElementById('file-input');
  
    // Preload sound files
    const userInputSound = new Audio('https://audio.jukehost.co.uk/Iov7lywB8bx3nqReTfH7vQ5k82wLbvfL.mp3');
    const aiResponseSound = new Audio('https://audio.jukehost.co.uk/W2A9Gwy4CHG6BKpl5ovAbYhrfT6VCOED.mp3');
  
    // Adjust volume (optional)
    userInputSound.volume = 0.5;
    aiResponseSound.volume = 0.5;
  
    // Function to send a text message
    async function sendMessage() {
      const message = userInput.value.trim();
      if (!message) return;
  
      // Disable input and button while processing
      userInput.disabled = true;
      sendButton.disabled = true;
  
      // Play user input sound
      userInputSound.currentTime = 0;
      userInputSound.play();
  
      // Display user message with typing animation
      await appendMessage(message, 'user', () => {
        userInput.value = ''; // Clear input field after animation
      });
  
      try {
        // Send message to the backend
        const response = await fetch('/api/all', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ message }),
        });
  
        const data = await response.json();
  
        // Display AI response with typing animation
        await appendMessage(data.response, 'bot', () => {
          // Play AI response sound after the typing animation is complete
          aiResponseSound.currentTime = 3;
          aiResponseSound.play();
  
          // Check if the response contains chart data
          if (data.chart) {
            renderChart(data.chart);
          }
        });
      } catch (error) {
        console.error('Error:', error);
        appendMessage('Sorry, there was an error processing your message.', 'bot');
      } finally {
        // Re-enable input and button
        userInput.disabled = false;
        sendButton.disabled = false;
        userInput.focus();
      }
    }
  
    // Function to handle file uploads
    async function handleFileUpload(file) {
      const reader = new FileReader();
      reader.onload = async (e) => {
        const fileContent = e.target.result;
  
        // Display uploaded file in the chat
        await appendMessage(`Uploaded file: ${file.name}`, 'user');
  
        try {
          // Send file to the backend for processing
          const response = await fetch('/api/all', {
            method: 'POST',
            headers: {
              'Content-Type': 'application/json',
            },
            body: JSON.stringify({ file: fileContent }),
          });
  
          const data = await response.json();
  
          // Display AI response with typing animation
          await appendMessage(data.response, 'bot', () => {
            // Play AI response sound after the typing animation is complete
            aiResponseSound.currentTime = 3;
            aiResponseSound.play();
  
            // Check if the response contains chart data
            if (data.chart) {
              renderChart(data.chart);
            }
          });
        } catch (error) {
          console.error('Error:', error);
          appendMessage('Sorry, there was an error processing your file.', 'bot');
        }
      };
      reader.readAsDataURL(file); // Read file as base64
    }
  
    // Function to append a message to the chat box with typing animation
    async function appendMessage(message, sender, onComplete) {
      const messageElement = document.createElement('div');
      messageElement.classList.add('message', sender);
  
      // Handle file uploads (base64 images)
      if (sender === 'user' && message.startsWith('Uploaded file:')) {
        const fileName = message.split(': ')[1];
        const fileType = fileName.split('.').pop().toLowerCase();
  
        if (['png', 'jpg', 'jpeg', 'gif'].includes(fileType)) {
          const img = document.createElement('img');
          img.src = message.split(',')[1]; // Extract base64 data
          img.alt = fileName;
          img.style.maxWidth = '100%';
          img.style.borderRadius = '8px';
          messageElement.appendChild(img);
        } else {
          messageElement.textContent = `Uploaded file: ${fileName}`;
        }
      } else {
        // Add an empty span to hold the text
        const textSpan = document.createElement('span');
        messageElement.appendChild(textSpan);
        chatBox.appendChild(messageElement);
  
        // Simulate typing effect
        await typeText(textSpan, message);
      }
  
      chatBox.appendChild(messageElement);
      chatBox.scrollTop = chatBox.scrollHeight; // Auto-scroll to the bottom
  
      // Execute the callback (if provided)
      if (onComplete) {
        onComplete();
      }
    }
  
    // Function to simulate typing animation
    async function typeText(element, text) {
      return new Promise((resolve) => {
        let i = 0;
        const typingSpeed = 10; // Delay between characters (in milliseconds)
        const punctuationDelay = 150; // Extra delay for punctuation marks
  
        function type() {
          if (i < text.length) {
            const currentChar = text.charAt(i);
  
            // Append the current character
            element.textContent += currentChar;
  
            // Check if the current character is a punctuation mark
            if (/[.,!?]/.test(currentChar)) {
              setTimeout(type, punctuationDelay); // Longer delay for punctuation
            } else {
              setTimeout(type, typingSpeed); // Regular delay for other characters
            }
  
            i++; // Move to the next character
          } else {
            resolve(); // Resolve the promise when typing is complete
          }
        }
  
        type(); // Start the typing animation
      });
    }
  
    // Function to render a chart using Chart.js
    function renderChart(chartData) {
      const chartContainer = document.createElement('div');
      chartContainer.classList.add('chart-container');
      chatBox.appendChild(chartContainer);
  
      const canvas = document.createElement('canvas');
      chartContainer.appendChild(canvas);
  
      const ctx = canvas.getContext('2d');
      new Chart(ctx, {
        type: chartData.type || 'bar', // Default to bar chart
        data: chartData.data,
        options: chartData.options || {
          responsive: true,
          maintainAspectRatio: false,
        },
      });
    }
  
    // Event listener for the send button
    sendButton.addEventListener('click', sendMessage);
  
    // Event listener for the Enter key in the input field
    userInput.addEventListener('keypress', (e) => {
      if (e.key === 'Enter') {
        sendMessage();
      }
    });
  
    // Event listener for file uploads
    fileInput.addEventListener('change', (e) => {
      const file = e.target.files[0];
      if (file) {
        handleFileUpload(file);
      }
    });
  });