document.addEventListener('DOMContentLoaded', () => {
    // Initialize the Markov chain model
    let markovModel = new MarkovChain(2);
    let isConnectedToDb = false;
    
    // Initialize with default training text
    const defaultText = document.getElementById('training-text').value;
    markovModel.train(defaultText);
    
    // DOM elements
    const chatMessages = document.getElementById('chat-messages');
    const messageInput = document.getElementById('message-input');
    const sendButton = document.getElementById('send-button');
    const trainButton = document.getElementById('train-button');
    const modelStats = document.getElementById('model-stats') || document.createElement('div');
    
    // Socket.io connection
    let socket;
    try {
        // Connect to the server
        socket = io(); // Connects to the host that served the page
        
        // Handle connection status
        socket.on('connect', () => {
            console.log('Connected to server');
            addSystemMessage('Connected to server');
        });
        
        socket.on('disconnect', () => {
            console.log('Disconnected from server');
            addSystemMessage('Disconnected from server. Trying to reconnect...');
        });
        
        // Handle incoming messages
        socket.on('message', (data) => {
            addMessage(data.text, false);
        });
        
        // Handle model configuration from server
        socket.on('modelConfig', (config) => {
            document.getElementById('model-order').value = config.order;
            isConnectedToDb = config.isDbConnected;
            if (isConnectedToDb) {
                addSystemMessage('Connected to database. Your messages will be stored and learned from.');
            } else {
                addSystemMessage('Running in memory-only mode. Your messages will not be persisted.');
            }
            updateModelStats();
        });
        
        // Handle training completion notification
        socket.on('trainComplete', (result) => {
            if (result.success) {
                addSystemMessage('Model trained successfully!');
            } else {
                addSystemMessage(`Training failed: ${result.error}`);
            }
            updateModelStats();
        });
    } catch (error) {
        console.error('Socket.io connection failed:', error);
        addSystemMessage('Could not connect to server. Running in local mode.');
    }
    
    // Function to update model stats display
    function updateModelStats() {
        if (!socket || !socket.connected) return;
        
        fetch('/api/model/stats')
            .then(response => response.json())
            .then(stats => {
                const statsContent = `
                    <div class="stats-container">
                        <div>Order: ${stats.order}</div>
                        <div>Transitions: ${stats.transitionCount.toLocaleString()}</div>
                        <div>Start Sequences: ${stats.startSequenceCount.toLocaleString()}</div>
                        <div>Database: ${stats.isDbConnected ? 'Connected' : 'Not connected'}</div>
                    </div>
                `;
                
                if (document.getElementById('model-stats')) {
                    document.getElementById('model-stats').innerHTML = statsContent;
                } else {
                    const statsDiv = document.createElement('div');
                    statsDiv.id = 'model-stats';
                    statsDiv.innerHTML = statsContent;
                    document.querySelector('.settings-panel').appendChild(statsDiv);
                }
            })
            .catch(error => {
                console.error('Error fetching model stats:', error);
            });
    }
    
    // Add a message to the chat
    function addMessage(text, isUser = false) {
        const messageDiv = document.createElement('div');
        messageDiv.classList.add('message');
        messageDiv.classList.add(isUser ? 'user-message' : 'bot-message');
        messageDiv.textContent = text;
        
        chatMessages.appendChild(messageDiv);
        chatMessages.scrollTop = chatMessages.scrollHeight;
    }
    
    // Add a system message (gray, centered)
    function addSystemMessage(text) {
        const messageDiv = document.createElement('div');
        messageDiv.classList.add('message');
        messageDiv.style.alignSelf = 'center';
        messageDiv.style.background = 'rgba(100, 100, 100, 0.2)';
        messageDiv.style.fontSize = '0.8rem';
        messageDiv.textContent = text;
        
        chatMessages.appendChild(messageDiv);
        chatMessages.scrollTop = chatMessages.scrollHeight;
    }
    
    // Send a message
    function sendMessage() {
        const messageText = messageInput.value.trim();
        if (messageText === '') return;
        
        // Add user message to chat
        addMessage(messageText, true);
        
        // Clear input field
        messageInput.value = '';
        
        // If connected to a server, send the message
        if (socket && socket.connected) {
            socket.emit('message', { text: messageText });
        } else {
            // In local mode, just use the local model
            setTimeout(() => {
                const maxLength = parseInt(document.getElementById('max-length').value);
                const response = markovModel.respond(messageText, maxLength);
                addMessage(response, false);
            }, 500);
        }
    }
    
    // Event listeners
    sendButton.addEventListener('click', sendMessage);
    
    messageInput.addEventListener('keypress', (e) => {
        if (e.key === 'Enter') {
            sendMessage();
        }
    });
    
    trainButton.addEventListener('click', () => {
        const trainingText = document.getElementById('training-text').value;
        const modelOrder = parseInt(document.getElementById('model-order').value);
        
        if (socket && socket.connected) {
            // Send training request to server
            socket.emit('train', {
                text: trainingText,
                order: modelOrder
            });
            addSystemMessage('Training request sent to server...');
        } else {
            // Update the model order if changed
            if (markovModel.order !== modelOrder) {
                markovModel = new MarkovChain(modelOrder);
            }
            
            // Train the model locally
            markovModel.train(trainingText);
            addSystemMessage('Model trained successfully in local mode!');
        }
    });
    
    // Add model stats section to the DOM
    if (!document.getElementById('model-stats')) {
        const statsContainer = document.createElement('div');
        statsContainer.id = 'model-stats';
        statsContainer.classList.add('model-stats');
        document.querySelector('.settings-panel').appendChild(statsContainer);
    }
    
    // Initialize stats if connected
    setTimeout(updateModelStats, 1000);
    
    // Initial system message
    addSystemMessage('Markov Chain LLM initialized. Type a message to start the conversation.');
});