/**
 * MarkovChain - A simple text generator based on Markov chains
 * This class creates a model from input text and can generate new text based on the patterns it learns
 */
/**
 * MarkovChain - A database-backed text generator based on Markov chains
 * This class creates a model from input text and can generate new text based on the patterns it learns
 */
class MarkovChain {
    constructor(order = 2, dbClient = null) {
        this.order = order; // The "memory" of the model - how many previous words to consider
        this.model = {}; // The transition probabilities (in-memory cache)
        this.startSequences = []; // Possible starting sequences
        this.dbClient = dbClient; // Database client for persistence
    }

    /**
     * Tokenize the input text into words
     * @param {string} text - The text to tokenize
     * @returns {Array} - Array of tokens (words)
     */
    tokenize(text) {
        // Basic tokenization: split by whitespace and keep punctuation
        return text.split(/\s+/).filter(token => token.length > 0);
    }

    /**
     * Train the model on the given text
     * @param {string} text - The training text
     * @param {boolean} saveToDb - Whether to save the model to the database
     */
    async train(text, saveToDb = true) {
        // Reset in-memory model
        this.model = {};
        this.startSequences = [];

        // Tokenize the text
        const tokens = this.tokenize(text);
        
        // If we don't have enough tokens for the selected order, reduce the order
        if (tokens.length < this.order + 1) {
            this.order = Math.max(1, tokens.length - 1);
            console.warn(`Not enough tokens for selected order. Reduced to ${this.order}.`);
        }

        // Build the model
        const transitions = [];
        const startSeqs = [];

        for (let i = 0; i <= tokens.length - this.order; i++) {
            // Get the current sequence (the "state")
            const sequence = tokens.slice(i, i + this.order).join(' ');
            
            // Get the next token after this sequence
            const nextToken = i + this.order < tokens.length ? tokens[i + this.order] : null;
            
            // If this is the start of a sentence, add it to possible starting sequences
            if (i === 0 || /[.!?]$/.test(tokens[i - 1])) {
                this.startSequences.push(sequence);
                if (saveToDb) {
                    startSeqs.push({ sequence });
                }
            }

            // Add this transition to the model
            if (nextToken) {
                if (!this.model[sequence]) {
                    this.model[sequence] = [];
                }
                this.model[sequence].push(nextToken);
                
                if (saveToDb) {
                    transitions.push({
                        sequence,
                        nextToken,
                        order: this.order
                    });
                }
            }
        }

        // If we found no starting sequences, just use the first sequence
        if (this.startSequences.length === 0 && tokens.length >= this.order) {
            const firstSequence = tokens.slice(0, this.order).join(' ');
            this.startSequences.push(firstSequence);
            if (saveToDb) {
                startSeqs.push({ sequence: firstSequence });
            }
        }

        // Save to database if requested and dbClient is available
        if (saveToDb && this.dbClient) {
            try {
                await this.saveTransitionsToDb(transitions);
                await this.saveStartSequencesToDb(startSeqs);
                console.log('Model saved to database successfully');
            } catch (error) {
                console.error('Error saving model to database:', error);
            }
        }

        return this;
    }

    /**
     * Save transition probabilities to the database
     * @param {Array} transitions - Array of transition objects
     */
    async saveTransitionsToDb(transitions) {
        if (!this.dbClient) return;
        
        try {
            // Use batch operations for efficiency
            const batchSize = 100;
            for (let i = 0; i < transitions.length; i += batchSize) {
                const batch = transitions.slice(i, i + batchSize);
                await this.dbClient.collection('transitions').insertMany(batch);
            }
        } catch (error) {
            console.error('Error saving transitions to database:', error);
            throw error;
        }
    }

    /**
     * Save starting sequences to the database
     * @param {Array} startSequences - Array of starting sequence objects
     */
    async saveStartSequencesToDb(startSequences) {
        if (!this.dbClient || startSequences.length === 0) return;
        
        try {
            await this.dbClient.collection('startSequences').insertMany(startSequences);
        } catch (error) {
            console.error('Error saving start sequences to database:', error);
            throw error;
        }
    }

    /**
     * Load the model from the database
     * @param {number} order - The order of the Markov model to load
     */
    async loadFromDb(order = null) {
        if (!this.dbClient) {
            console.error('No database client available');
            return this;
        }
        
        if (order) {
            this.order = order;
        }
        
        try {
            // Reset in-memory model
            this.model = {};
            this.startSequences = [];
            
            // Load transitions
            const transitions = await this.dbClient.collection('transitions')
                .find({ order: this.order })
                .toArray();
            
            // Build in-memory model from transitions
            for (const transition of transitions) {
                const { sequence, nextToken } = transition;
                if (!this.model[sequence]) {
                    this.model[sequence] = [];
                }
                this.model[sequence].push(nextToken);
            }
            
            // Load starting sequences
            const startSeqs = await this.dbClient.collection('startSequences').find({}).toArray();
            this.startSequences = startSeqs.map(item => item.sequence);
            
            console.log(`Loaded model from database with ${transitions.length} transitions and ${this.startSequences.length} starting sequences`);
        } catch (error) {
            console.error('Error loading model from database:', error);
        }
        
        return this;
    }

    /**
     * Generate a response based on the trained model
     * @param {number} maxLength - Maximum length of the generated text in words
     * @param {string} seed - Optional seed to start the generation
     * @returns {string} - Generated text
     */
    generate(maxLength = 100, seed = null) {
        // If the model is empty, return an error message
        if (Object.keys(this.model).length === 0) {
            return "The model hasn't been trained yet. Please train it first.";
        }

        // Start with a random sequence from the starting sequences or the provided seed
        let currentSequence;
        if (seed) {
            // Try to find a sequence that starts with the seed
            const seedTokens = this.tokenize(seed);
            const matchingSequences = Object.keys(this.model).filter(seq => 
                seq.toLowerCase().startsWith(seedTokens[0].toLowerCase())
            );
            
            if (matchingSequences.length > 0) {
                // Pick a random matching sequence
                currentSequence = matchingSequences[Math.floor(Math.random() * matchingSequences.length)];
            } else {
                // If no match, use a random starting sequence
                currentSequence = this.startSequences[Math.floor(Math.random() * this.startSequences.length)];
            }
        } else {
            // Use a random starting sequence
            currentSequence = this.startSequences[Math.floor(Math.random() * this.startSequences.length)];
        }

        // Initialize the result with the current sequence
        const result = currentSequence.split(' ');

        // Generate the text
        for (let i = 0; i < maxLength - this.order; i++) {
            // Get possible next tokens
            const possibleNextTokens = this.model[currentSequence];
            
            // If we hit a dead end, break
            if (!possibleNextTokens || possibleNextTokens.length === 0) {
                break;
            }
            
            // Choose a random next token
            const nextToken = possibleNextTokens[Math.floor(Math.random() * possibleNextTokens.length)];
            result.push(nextToken);
            
            // Update the current sequence by removing the first word and adding the new one
            const sequenceTokens = currentSequence.split(' ');
            sequenceTokens.shift();
            sequenceTokens.push(nextToken);
            currentSequence = sequenceTokens.join(' ');
            
            // If we hit end-of-sentence punctuation, consider stopping early with a small probability
            if (/[.!?]$/.test(nextToken) && Math.random() < 0.3 && i > maxLength / 2) {
                break;
            }
        }

        return result.join(' ');
    }

    /**
     * Generate a response that attempts to be relevant to the input
     * @param {string} input - The input message
     * @param {number} maxLength - Maximum length of the generated response
     * @returns {string} - Generated response
     */
    respond(input, maxLength = 100) {
        // Add the input to the training data
        this.train(input, true).catch(err => console.error('Error training on input:', err));
        
        // Try to use input as a seed if possible
        const tokens = this.tokenize(input);
        
        if (tokens.length === 0) {
            return this.generate(maxLength);
        }
        
        // Pick a random word from the input as a potential seed
        const seedIndex = Math.floor(Math.random() * tokens.length);
        const seed = tokens[seedIndex];
        
        return this.generate(maxLength, seed);
    }
}

// Export the class if in a Node.js environment
if (typeof module !== 'undefined' && typeof module.exports !== 'undefined') {
    module.exports = { MarkovChain };
}