const express = require('express');
const http = require('http');
const socketIo = require('socket.io');
const path = require('path');
const { MongoClient } = require('mongodb');
const { MarkovChain } = require('./markov');

// MongoDB connection string
const mongoUri = process.env.MONGO_URI || 'mongodb://localhost:27017/markovLLM';

// Initialize Express app
const app = express();
const server = http.createServer(app);
const io = socketIo(server);

// Serve static files
app.use(express.static(path.join(__dirname, 'public')));

// Default training text
const defaultText = `
Artificial intelligence is transforming how we interact with technology.
Language models use statistical patterns to generate text that mimics human writing.
Markov chains are a simple yet effective way to model language patterns.
While not as sophisticated as neural networks, they can still produce interesting results.
`;

let markovModel;
let dbClient;

// Connect to MongoDB and initialize the model
async function initializeDatabase() {
    try {
        // Connect to MongoDB
        dbClient = new MongoClient(mongoUri, { useNewUrlParser: true, useUnifiedTopology: true });
        await dbClient.connect();
        console.log('Connected to MongoDB');
        
        const db = dbClient.db();
        
        // Create indexes for better performance
        await db.collection('transitions').createIndex({ sequence: 1, order: 1 });
        await db.collection('startSequences').createIndex({ sequence: 1 });
        
        // Initialize the Markov chain with database connection
        markovModel = new MarkovChain(2, db);
        
        // Try to load existing model from database
        await markovModel.loadFromDb();
        
        // If model is empty, train with default text
        if (Object.keys(markovModel.model).length === 0) {
            console.log('No existing model found, training with default text');
            await markovModel.train(defaultText);
        }
        
        console.log('Markov model initialized successfully');
    } catch (error) {
        console.error('Failed to connect to MongoDB:', error);
        // Fallback to in-memory model if database connection fails
        markovModel = new MarkovChain(2);
        markovModel.train(defaultText);
    }
}

// Initialize database connection
initializeDatabase().catch(console.error);

// Socket.io connection handler
io.on('connection', (socket) => {
    console.log('Client connected:', socket.id);
    
    // Send the current model configuration to the client
    socket.emit('modelConfig', {
        order: markovModel ? markovModel.order : 2,
        isDbConnected: !!dbClient
    });
    
    // Handle incoming messages
    socket.on('message', async (data) => {
        console.log(`Message from ${socket.id}: ${data.text}`);
        
        try {
            // Learn from this message by adding it to the model
            if (markovModel) {
                await markovModel.train(data.text);
            }
            
            // Generate a response using the Markov model
            const response = markovModel ? markovModel.respond(data.text, 100) : 
                "Sorry, the model is not ready yet.";
            
            // Send response back to the client
            socket.emit('message', { text: response });
        } catch (error) {
            console.error('Error processing message:', error);
            socket.emit('message', { 
                text: "Sorry, I encountered an error while processing your message." 
            });
        }
    });
    
    // Handle model training requests
    socket.on('train', async (data) => {
        try {
            if (markovModel) {
                markovModel.order = parseInt(data.order) || 2;
                await markovModel.train(data.text);
                socket.emit('trainComplete', { success: true });
            } else {
                socket.emit('trainComplete', { 
                    success: false, 
                    error: "Model not initialized" 
                });
            }
        } catch (error) {
            console.error('Error training model:', error);
            socket.emit('trainComplete', { 
                success: false, 
                error: "Failed to train model" 
            });
        }
    });
    
    // Handle disconnection
    socket.on('disconnect', () => {
        console.log('Client disconnected:', socket.id);
    });
});

// Routes
app.get('/', (req, res) => {
    res.sendFile(path.join(__dirname, 'public', 'index.html'));
});

// API endpoint to get model stats
app.get('/api/model/stats', async (req, res) => {
    if (!markovModel) {
        return res.status(503).json({ error: "Model not initialized" });
    }
    
    try {
        const transitionCount = dbClient ? 
            await dbClient.db().collection('transitions').countDocuments() : 
            Object.values(markovModel.model).reduce((sum, arr) => sum + arr.length, 0);
        
        const startSequenceCount = markovModel.startSequences.length;
        
        res.json({
            order: markovModel.order,
            transitionCount,
            startSequenceCount,
            isDbConnected: !!dbClient
        });
    } catch (error) {
        console.error('Error getting model stats:', error);
        res.status(500).json({ error: "Failed to get model stats" });
    }
});

// Start the server
const PORT = process.env.PORT || 10000;
server.listen(PORT, () => {
    console.log(`Server running on http://thomaslopezforte.searchcontent.tech:${PORT}`);
});

// Graceful shutdown
process.on('SIGINT', async () => {
    console.log('Shutting down server...');
    if (dbClient) {
        await dbClient.close();
        console.log('Database connection closed');
    }
    process.exit(0);
});